package edu.udel.cisc474.scheduler.model;

public class Event {
	
	/* Days of the week */
	public static final int SUNDAY = 0;
	public static final int MONDAY = 1;
	public static final int TUESDAY = 2;
	public static final int WEDNESDAY = 3;
	public static final int THURSDAY = 4;
	public static final int FRIDAY = 5;
	public static final int SATURDAY = 6;
	
	/* Arrays to make it easier to derive a string representation of an event */
	public static final String[] DAYSOFWEEK = {"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday", "Saturday"};
	public static final String[] DAYABBREVIATIONS = {"S","M","T","W","R","F", "A"};
	
	/* No magic numbers here */
	public static final int DAYS_IN_WEEK = 7;
	public static final int MAX_TIME = 2359;
	
	private int startTime;
	private int endTime;
	private int dayOfWeek;
	
	public Event(){
		this(-1,-1,-1);
	}
	
	public Event(int startTime, int endTime, int dayOfWeek){
		setStartTime(startTime);
		setEndTime(endTime);
		setDayOfWeek(dayOfWeek);
	}
	
	/* returns true if the integer passed to it is a valid time */
	private boolean isTimeValid(int time){
	  return (time <= MAX_TIME && time >=0 && (time % 100) < 60);	
	}
	
	/* returns true if the event is valid */
	public boolean isValid(){
		return (isTimeValid(startTime) && isTimeValid(endTime) && dayOfWeek >=0 && dayOfWeek < DAYS_IN_WEEK && startTime < endTime);
	}


	public String toString(){
		return DAYABBREVIATIONS[this.getDayOfWeek()] + " " + displayTime(this.getStartTime()) + "-" + displayTime(this.getEndTime());
	}
	
	public String getTimeSpan(){
		return displayTime(startTime) + " - " + displayTime(endTime);
	}

	public String displayTime(int time){
		int min = time % 100;
		int hour = (time - min) / 100;
		String a = "am";
		
		if (hour > 12){
			hour = hour % 12;
			a = "pm";
		}
		if (hour == 12){
			a = "pm";
		}
		if (hour == 0){
			a = "am";
			hour = 12;
		}
		
		return hour + ":" + ((min < 10) ? "0" + min : min) + a;
	}
	
	public int getDayOfWeek() {
		return dayOfWeek;
	}

	public void setDayOfWeek(int dayOfWeek) {
		this.dayOfWeek = dayOfWeek;
	}

	public int getEndTime() {
		return endTime;
	}

	public void setEndTime(int endTime) {
		this.endTime = endTime;
	}

	public int getStartTime() {
		return startTime;
	}
	
	public boolean equals(Event e){
		return (this.compareTo(e) == 0);
	}

	public void setStartTime(int startTime) {
		this.startTime = startTime;
	}
	
	public Event overlapEvents(Event e){
		if (this.getDayOfWeek() == e.getDayOfWeek()){
			//if start time & end time of 1 surround a start, end or both of the other
			if (this.getStartTime() <= e.getStartTime() && this.getEndTime() >= e.getStartTime()){
				return new Event(this.getStartTime(),Math.max(this.getEndTime(),e.getEndTime()), this.getDayOfWeek());
			}
			if (this.getStartTime() >= e.getStartTime() && this.getStartTime() <= e.getEndTime()){
				return new Event(e.getStartTime(),Math.max(this.getEndTime(),e.getEndTime()), this.getDayOfWeek());
			}
		}
		return null; // non-overlapping
	}
	
	public int compareTo(Event e){
		if (this.getDayOfWeek() == e.getDayOfWeek()){
			if (this.getStartTime() == e.getStartTime()){
				if (this.getEndTime() == e.getEndTime()){
					return 0;
				}else{
					return (this.getEndTime() < e.getEndTime()) ? -1 : 1;
				}
			}else{
				return (this.getStartTime() < e.getStartTime()) ? -1 : 1;
			}
		}else{
			return (this.getDayOfWeek() < e.getDayOfWeek()) ? -1 : 1;
		}
	}
	


}
