package edu.udel.cisc474.scheduler.model;

import java.util.LinkedList;

public class Schedule {

	/* Structure to store event list */
	private LinkedList<Event> events;
	
	private int[] firstEventOfDay;
	
	/* constructs an empty schedule object */
	public Schedule(){
		events = new LinkedList<Event>();
		firstEventOfDay = new int[Event.DAYS_IN_WEEK];
		for (int i = 0; i < Event.DAYS_IN_WEEK; i++){
			firstEventOfDay[i] = -1;
		}
	}
	
	/* constructs a schedule object from a string representation of an events */	
	public Schedule(String schedule){
		this();
		LinkedList<Event> tmpE = ScheduleUtils.parseSchedule(schedule);

		for (int i=0; i < tmpE.size(); i++){
			System.out.println("Adding " + tmpE.get(i));
			addEvent(tmpE.get(i));
		}
	}
	
	public void addEvent(Event e){
		int position = 0;
		while (position < events.size() && events.get(position).compareTo(e) < 0){
			position++;
		}

		events.add(position,e);
		int eventDay = e.getDayOfWeek();
		
		/* if the first event for this day is after this, set this as the first event */
		if (firstEventOfDay[eventDay] > position || firstEventOfDay[eventDay] < 0){
			firstEventOfDay[eventDay] = position;
		}
		
		/* if an event is added before another event, increase first event for day */
		for (int i = eventDay+1; i < Event.DAYS_IN_WEEK; i++){
			if (firstEventOfDay[i] != -1){
				firstEventOfDay[i]++;
			}
		}
		combineOverlappingEvents();

	}

	public Schedule getFreeTimes(){
		Schedule s = new Schedule();
		Event t1;
		Event t2;
		for (int i = 0; i < Event.DAYS_IN_WEEK; i++){
			if (firstEventOfDay[i] < 0){
				s.addEvent(new Event(0,Event.MAX_TIME, i));
			}else{
				t1 = getEvent(firstEventOfDay[i]);
				t2 = getEvent(getEndOfDay(i));
				if (t1.getStartTime() > 0){
					s.addEvent(new Event(0,t1.getStartTime(), i));
				}
				if (t2.getEndTime() < Event.MAX_TIME){
					s.addEvent(new Event(t2.getEndTime(),Event.MAX_TIME, i));
				}
				
			}
			
			for (int j = firstEventOfDay[i]; j < getEndOfDay(i); j++){
				Event tmp1 = getEvent(j);
				Event tmp2 = getEvent(j+1);
				s.addEvent(new Event(tmp1.getEndTime(),tmp2.getStartTime(),i));
			}
		}
		
		
		return s;
		
	}
	
	public void printStarts(){
		for (int i = 0; i < Event.DAYS_IN_WEEK; i++){
			System.out.println(Event.DAYSOFWEEK[i] + ": " + firstEventOfDay[i] + " through " + getEndOfDay(i));
		}
	}
	
	private void combineOverlappingEvents(){
		int i = 0;
		Event tmpEvent1 = null;
		Event tmpEvent2 = null;
		Event overlap = null;
		while (i < events.size() - 1){
			tmpEvent1 = events.get(i);
			tmpEvent2 = events.get(i + 1);
			overlap = tmpEvent1.overlapEvents(tmpEvent2);
			if (overlap != null){
				events.remove(tmpEvent1);
				events.remove(tmpEvent2);
				events.add(i,overlap);
			}else{
				i++;
			}
		}
	}
	
	public Event getEvent(int index){
		return events.get(index);	
	}

	private int getEndOfDay(int day){
		if (firstEventOfDay[day] < 0){
			return -1;
		}
		for (int i = day+1; i < Event.DAYS_IN_WEEK; i++){
			if (firstEventOfDay[i] > -1){
				return firstEventOfDay[i] - 1;
			}
		}
		return size() - 1;
	}
	
	public int size(){
		return events.size();
	}
	
	public void addSchedule(Schedule s){
		for (int i = 0; i < s.size(); i++){
			Event e = s.getEvent(i); 
			addEvent(new Event(e.getStartTime(), e.getEndTime(), e.getDayOfWeek()));
		}
	}
	
	public static void main(String args[]){
		Schedule s = new Schedule("M 10:00pm-11pm, M 1:00am-2am, W 9am-5pm, R 6am-3:30pm, ");
		for (int i = 0; i < s.size(); i++){
			System.out.println(s.getEvent(i));
		}
		s.printStarts();
		
		Schedule s2=s.getFreeTimes();
		for (int i = 0; i < s2.size(); i++){
			System.out.println(s2.getEvent(i));
		}
	}
	
}
