package edu.udel.cisc474.scheduler.model;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.LinkedList;

public class ScheduleUtils {

	/* once again, no magic numbers here */
	private static final int days=1;
	private static final int startHour=2;
	private static final int startMin=5;
	private static final int startAMPM=6;
	private static final int endHour=7;
	private static final int endMin=10;
	private static final int endAMPM=11;
	
	public static LinkedList<Event> parseSchedule(String schedule){
		
		LinkedList<Event> retVal = new LinkedList<Event>();

		String[] scheduleEvents = schedule.split(",");
		String scheduleRegex = "^([SMTWRFA]{1,7}) (\\d{1,2})((:)(\\d{2})){0,1}([ap]m)-(\\d{1,2})((:)(\\d{2})){0,1}([ap]m)$";
		Pattern schedulePattern = Pattern.compile(scheduleRegex);
		Matcher matcher;

		for (int i = 0; i < scheduleEvents.length; i++){
			matcher = schedulePattern.matcher(scheduleEvents[i].trim());
			if (matcher.find()){
				for (int j = 0; j < matcher.group(days).length(); j++){
					int startTime = makeTime(matcher.group(startHour),matcher.group(startMin),matcher.group(startAMPM));
					int endTime = makeTime(matcher.group(endHour),matcher.group(endMin),matcher.group(endAMPM));
					int thisDay = makeDay(matcher.group(1).charAt(j));
					Event e = new Event(startTime, endTime, thisDay);
					retVal.add(e);
				}
			}else{
				//badInput 
			}

		}
		return retVal;
	}
		
	private static int makeTime(String hour, String min, String ampm){
		int retVal = 0;
		retVal += Integer.parseInt(hour) * 100;

		if (ampm.equals("pm") && retVal < 1200){
			retVal += 1200;
		}

		if (ampm.equals("am") && retVal == 1200){
			retVal -= 1200;
		}


		if (min != null){
			retVal += Integer.parseInt(min);
		}
		return retVal % 2400;
	}

	private static int makeDay(char day){
		switch (day){
			case 'S':
				return Event.SUNDAY;
			case 'M':
				return Event.MONDAY;
			case 'T':
				return Event.TUESDAY;
			case 'W':
				return Event.WEDNESDAY;
			case 'R':
				return Event.THURSDAY;
			case 'F':
				return Event.FRIDAY;
			case 'A':
				return Event.SATURDAY;
		}
		return -1;
	} 
	
}
